package edu.stanford.nlp.framenet;

import java.util.*;
import edu.stanford.nlp.trees.*;

/** A layer of FrameNet annotation corresponds to a single target and
 * the annotation associated with it.
 *
 * @author Roger Levy
 * @author Cindi Thompson
 */
public class FNLayer {

  private FNFrame frame;
  private Collection roles;

  /** since a target can be multi-word, make it a list.  (Can't be
   * discontinuous!)  */
  private Sentence target;

  /** Constructor with empty frame name. */
  public FNLayer() {
    this("");
  }

  /** Constructor whose frame is the given name. */
  public FNLayer(String name) {
    frame = new FNFrame(name);
    roles = new HashSet();
  }

  public FNFrame frame() {
    return frame;
  }

  public void setFrame(String name) {
    frame.setName(name);
  }
  
  public void addRole(String name, String words) {
    roles.add(new FNRole(name, words));
  }

  private FNRole addRole(String name, String words, int start, int end) {
    FNRole role= new FNRole(name, words);
    role.setStart(start);
    role.setEnd(end);
    roles.add(role);
    return role;
  }

  /** Add the name and start/end positions to a role; creates the role 
   * if it is not yet in the Layer.
   */
  public void addFERole(String name, String words, int start, int end) {
    FNRole role = roleExists(start, end);
    if (role ==null)
      addRole(name, words, start, end);
    else {
      role.setRoleName(name);
      role.setWords(words);
    }
  }//addFERole

  /** checks whether a role exists with given start and end pos in sent. 
   *  and returns it if so.
   */
  private FNRole roleExists(int start, int end) {
    boolean found = false;
    FNRole role=new FNRole();
    FNRole next;
    for (Iterator i=roles.iterator(); i.hasNext() &&!found ; ) {
      next = (FNRole)i.next();
      if (next.getStart()==start && next.getEnd()==end) {
	found = true;
	role = next;
      }
    }
    if (found)
      return role;
    else 
      return null;
  }//roleExists
	
  /** add PT or GT info; role may not exist yet, if not then create it.
   * type indicates whether PT or GF.
   * should probably really throw some exceptions upon error.
   */
  public void updateRole(int start, int end, String type, String label) {
    //search for role that starts and ends at the right place
    String roleName="";
    FNRole role = roleExists(start, end);
    if (role == null)
      role = addRole("","",start,end);
    if (type.equals("PT"))
      role.setPT(label);
    else if (type.equals("GF"))
      role.setGF(label);
    else
	System.out.println("unknown type: "+type+" NOT set to: "+label);
  }//updateRole

  public FNRole[] getRoles() {
    FNRole[] r = new FNRole[roles.size()];
    int n = 0;
    for(Iterator i = roles.iterator(); i.hasNext(); ){
      r[n] = (FNRole) i.next();
      n++;
    }
    return r;
  }
  
  public Sentence getTarget() {
    return target;
  }

  public void setTarget(String targetwords) {
    StringTokenizer st = new StringTokenizer(targetwords);
    Collection eachWord = new Vector();
    while (st.hasMoreTokens())
      eachWord.add(new Word(st.nextToken()));
    target = new Sentence(eachWord);
  }

  public String toString() {
    String ret ="frame: "+frame.name()+" target: "+target.toString()+"\n roles:\n";
    for (Iterator i=roles.iterator(); i.hasNext(); ) {
      ret += "       "+i.next().toString()+"\n";
    }
    return ret+"\n";
  }

}//FNLayer class
